<?php

/**
 * Extension to Payment  using Mellat Bank accounts
 * Based on Mohammad Amini code <http://www.yiiframework.com/user/99958> for yii-RezvanPayment
 *
 * @author faravaghi <faravaghi@gmail.com>
 * @license MIT
 * @version 1.0.1
 */

/**
 * How to use
 *
 * You at least need PHP 5.3+ adn cURL.
 *
 * Change your config/main.php to add a new component:
 *
 * 'components' => array
 *(
 *     'Payment' => array
 *    (
 *         'class'    		=> 'ext.RezvanPayment.RezvanPayment',
 *         'MerchantID'     => 'user to login',
 *         'Password'   => 'password to login',	// password to web login
 *     )
 * )
 *
 */
class RezvanPayment extends CApplicationComponent
{
	/**
	 * Terminal Id
	 * @var string
	 */
	public $MerchantID;

	/**
	 * User Password
	 * @var string
	 */
	public $Password;

	/**
	 * Api URL
	 * Api Url to call without 'http'
	 * @var string
	 */
	public $urlRequestToken = 'https://sep.shaparak.ir/Payments/InitPayment.asmx?WSDL';
	public $urlverifyTransaction = 'https://acquirer.samanepay.com/payments/referencepayment.asmx?WSDL';
	public $urlreverseTransaction = 'https://acquirer.samanepay.com/payments/referencepayment.asmx?WSDL';

	/**
	 * Api URL
	 * Api Url to call without 'http'
	 * @var string
	 */
	public $urlPay = 'https://sep.shaparak.ir/Payment.aspx';

	/**
	 * namespace
	 * @var string
	 */
	public $namespace='http://interfaces.core.sw.bps.com/';

	/**
	 * Server Response CODE
	 * @var array
	 */
	protected $client;

	/**
	 * Initialization
	 *
	 */
	public function init()
	{
		require(dirname(__FILE__) . "/lib/nusoap.php");

		parent::init( );

		Yii::trace( 'Extension initializating', 'RezvanPayment' );
	}

	/**
	 * List of API calls
	 *
	 * @return array
	 */
	protected function calls( )
	{
		return array
		(
			'RequestToken'  => array(
				'required' => array(
					'ResNUM',
					'TotalAmount',
				),
				'optional' => array(
					'SegAmount1',
					'SegAmount2',
					'SegAmount3',
					'SegAmount4',
					'SegAmount5',
					'SegAmount6',
					'AdditionalData1',
					'AdditionalData2',
					'Wage',
				),
				'type'=>array(
					'string',
					'integer',
					'integer',
					'integer',
					'integer',
					'integer',
					'integer',
					'integer',
					'string',
					'string',
					'integer',
				)
			),

			'verifyTransaction' => array(
				'required' => array(
					'RefNum',
				),
				'type'=>array(
					'string',
				)
			),

			'reverseTransaction' => array(
				'required' => array(
					'RefNum',
				),
				'type'=>array(
					'string',
				)
			),
		);
	}

	/**
	 * Process API calls
	 *
	 * @param string $call Call name in camelCase
	 * @param array $args Arguments to convert the API call array
	 * @return mixed Response to API call in accordance
	 */
	public function __call($call, $args)
	{
		$calls = $this->calls();
		$callName = $call;

		if(!isset($calls[$callName]))
			throw new CException( 'Unknown call' );

		$params = $calls[$callName];

		$params['required'] = isset( $params['required'] ) ?(array) $params['required'] : array();
		$params['optional'] = isset( $params['optional'] ) ?(array) $params['optional'] : array();
		$params['fixed'] = isset($params['fixed']) ? $params['fixed'] : array();
		$params['type'] = isset($params['type']) ? $params['type'] : array();

		$args = $this->populate($args, $params['required'], $params['optional'], $params['type']);
		$args = array_merge( $args, $params['fixed'] );

		$this->SOAPServer($callName);
		$response = $this->request( $callName, $args );

		if(isset($response))
		{
			if(is_array($response))
			{
				return array('error'=>true,'response'=> $response['faultstring']);
			}

			if($callName == 'RequestToken')
			{
				// if response is negative number that has error in request
				// and Must show fault else return token
				if(!preg_match("/^-\d*\.?\d+$/i", $response))
					return array('error'=>false,'response'=>$response);
				else
					return array('error'=>true,'response'=>Yii::t('RezvanPayment.rezvan', $response));
			}
			else
			{
				if($response < 0)
					return array('error'=>true,'response'=>Yii::t('RezvanPayment.rezvan', $response));
				else
					return array('error'=>false,'response'=>$response);
			}
		}
		else
			throw new CException( 'Response not found' );
	}

	/*
	* Formation of an array of call parameters of the arguments
	* @ Param mixed $ args The arguments passed to the method call
	* @ Param array $ required array parameters required
	* @ Param array $ optional array of additional parameters
	* @ Return array An array of call parameters
	**/
	protected function populate( $args, $required, $optional, $type )
	{
		$allParams = array_merge( $required, $optional );

		// Special case
		if( count( $args ) === 1 )
			// Check if the first argument is an array
			if( is_array( $args[0] ) )
				// ...And if all keys - line
				if( array_reduce( array_keys( $args[0] ), function( $prev, $curr ) { return $prev && is_string( $curr ); }, true ) )
				{
					foreach( $required as $requiredParam )
						if( ! isset( $args[0][$requiredParam] ) )
							throw new CException( 'It lacks some parameters' );

					return $args[0];
				}

		if( count( $args ) < count( $required ) )
			throw new CException( 'It lacks some required parameteres' );

		if( count( $args ) > count( $allParams ) )
			throw new CException( 'Too much parameteres' );

		$params = array( );

		foreach( $args as $id => $arg ){
			if(isset($type[$id]))
				settype($arg, $type[$id]);
			
			$params[$allParams[$id]] = $arg;
		}

		return $params;
	}

	/**
	 * API Call
	 * @param string $call name to call
	 * @param array $params call parameteres
	 * @return array server response
	 */
	protected function request( $call, array $params = array( ) )
	{
		switch ($call) {
			case 'RequestToken':
				$params = array_merge( array( 'TermID' => $this->MerchantID) ,$params);
				break;

			case 'verifyTransaction':
				$_param = array_merge($params, array( 'MerchantID' => (string)$this->MerchantID));
				$params = array();
				foreach ($_param as $key => $value) {
					$params[] = $value;
				}
				break;

			case 'reverseTransaction':
				$_param = array_merge($params , array(
					'MID' => $this->MerchantID,
					'Username' => $this->MerchantID,
					'Password' => $this->Password
				));
				$params = array();
				foreach ($_param as $key => $value) {
					$params[] = $value;
				}
				break;
		}

		$response = $this->client->call($call, $params);

		return $response;
	}

	/**
	 * Creating the SOAP server
	 * @param string $call name to call
	 * @param array $params call parameteres
	 * @return array server response
	 */
	protected function SOAPServer($call)
	{
		$_url = 'url' . $call ;

		$this->client = new nusoap_client($this->$_url, 'wsdl', '', '', '', '');
		$this->client->soap_defencoding = 'UTF-8';
		$this->client->decode_utf8 = false;

		if ($this->client->fault) {
			trigger_error("SOAP Fault: (faultcode: {$result->faultcode}, faultstring: {$result->faulstring})", E_ERROR);
		} else {
			$error = $this->client->getError();
			if ($error) {
				throw new CException($error);
			}
		}
	}
}
